/**
 * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
 *                                             ,B" ]L,,p%%%,,,§;, "K
 * Copyright (C) 2021 Mandelbulber Team        §R-==%w["'~5]m%=L.=~5N
 *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
 * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
 *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
 * Mandelbulber is free software:     §R.ß~-Q/M=,=5"v"]=Qf,'§"M= =,M.§ Rz]M"Kw
 * you can redistribute it and/or     §w "xDY.J ' -"m=====WeC=\ ""%""y=%"]"" §
 * modify it under the terms of the    "§M=M =D=4"N #"%==A%p M§ M6  R' #"=~.4M
 * GNU General Public License as        §W =, ][T"]C  §  § '§ e===~ U  !§[Z ]N
 * published by the                    4M",,Jm=,"=e~  §  §  j]]""N  BmM"py=ßM
 * Free Software Foundation,          ]§ T,M=& 'YmMMpM9MMM%=w=,,=MT]M m§;'§,
 * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
 * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
 * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
 *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
 * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
 * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
 * but WITHOUT ANY WARRANTY;                            .'''
 * without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Mandelbulber. If not, see <http://www.gnu.org/licenses/>.
 *
 * ###########################################################################
 *
 * Authors: Krzysztof Marczak (buddhi1980@gmail.com)
 *
 * TODO: description
 */

#ifndef MANDELBULBER2_QT_LIGHT_WIDGET_H_
#define MANDELBULBER2_QT_LIGHT_WIDGET_H_

#include <memory>

#include <QWidget>

#include "src/algebra.hpp"
#include "src/camera_target.hpp"
#include "src/color_structures.hpp"

class cLightWidget : public QWidget
{
	Q_OBJECT
public:
	cLightWidget(QWidget *parent = nullptr);
	void SetLightAngle(CVector3 _angle);
	void SetLightColor(sRGBFloat _color);
	void SetCameraTarget(CVector3 camera, CVector3 target, CVector3 top);
	void SetRelativeMode(bool relativeRotationMode);
	void Render();

private:
	void paintEvent(QPaintEvent *event) override;
	void mousePressEvent(QMouseEvent *event) override;
	void mouseReleaseEvent(QMouseEvent *event) override;
	void mouseMoveEvent(QMouseEvent *event) override;
	void changeEvent(QEvent *event) override;

	CVector3 lightAngle;
	sRGBFloat lightColor;
	cCameraTarget cameraTarget;
	bool relativeRotationMode = false;
	std::vector<sRGBA8> image8;
	int size = 0;

	bool draggingInitStarted = false;
	bool draggingStarted = false;
	CVector3 lightAngleBeforeDrag;
	CVector2<int> dragStartPosition;

signals:
	void angleChanged(double alpha, double beta);
};

#endif /* MANDELBULBER2_QT_LIGHT_WIDGET_H_ */
